/*
 * Photo Image Print System
 * Copyright (C) 2001-2004 EPSON KOWA Corporation.
 * Copyright (C) SEIKO EPSON CORPORATION 2001-2004.
 *
 * This file is part of the `ekpstm' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "interface.h"
#include "support.h"

#include "stm.h"
#include "ekpcom.h"
#include "inkbox.h"
#include "error.xpm"
#include "error_i3.xpm"
#include "error_i4.xpm"
#include "error_i6.xpm"
#include "error_i8.xpm"
#include "error_ia.xpm"
#include "printing1.xpm"
#include "printing2.xpm"
#include "printing3.xpm"
#include "ready.xpm"
#include "ready_i1.xpm"
#include "warning.xpm"

typedef enum
{
	STS_ERR_FATAL,
	STS_ERR_JAM,
	STS_ERR_NO_INK,
	STS_ERR_NO_PAPER,
	STS_ERR_PRINTER,
	STS_NOT_READY,
	STS_PRINTING,
	STS_IDLE,
	STS_NO_REPLY,
	STS_MOTOR_OFF,		/* add sk     -Fri Mar  9 2001 */
	STS_EKPD_DOWN,		/* add sk     -Mon Mar 12 2001 */
	STS_ERR_PART_CHANGE,
	STS_ERR_ROLL,
	STS_ERR_REPETITION,
	STS_PRINTER_RESET,
	STS_DEFAULT
}PRINTER_STS;

typedef struct{
	GtkWidget *window;
	GtkWidget *prt_pix;

	PRINTER_STS cur_status;
	gint num_pict;
	gint cur_pict;
	gint timer;
	GdkPixmap *pixmap[4];
	GdkBitmap *mask[4];
	gchar **inkbox_xpm;
	gint inkbox_use_flag[INK_NUMBER];
	gint inkbox_counter;
	int stm_st;			/* ơ˥ȤΥơե饰 */
}PRINTSTSSTRUCT;

static PRINTSTSSTRUCT _status_str;


static PRINTER_STS ReadStatuslogfile (gint*);
static void SetAllUserButtonHidden (void);
static void DisplayInformation (GtkText*, char*);
static int GetPrinterStatus (gpointer);
static void auto_down_count_start (void);
static void auto_down_count (int);
static void setInkProg(gint *);

#define OFFSET_SEVENTH_INK_CARTRIDGE 34

static PRINTER_STS
ReadStatuslogfile (gint *rests)
{
	char *lpInk, *lpSts, *lpErr;
	char lpReply[1024], StsCode[3], ErrCode[3];
	char get_status_command[] = { 's', 't', 0x01, 0x00, 0x01 };
	int com_len, rep_len, i;
  
	/*  */
	StsCode[2] = 0;
	ErrCode[2] = 0;
	for (i = 0; i < INK_NUMBER; i++)
		rests[i] = -1;


	com_len = sizeof (get_status_command);
	rep_len = 1024;
	/* եɤ߹ */
	if (sock_write (get_status_command, &com_len) < 0)
	{
		if (sock_reopen () < 0)
		{
			return STS_EKPD_DOWN;
		}
		return STS_NO_REPLY;
	}
  
	sock_read (lpReply, &rep_len);
	if (rep_len == 0)
	{
		return STS_NO_REPLY;
	}
	/* 󥯻̲ */
	lpInk = strstr(lpReply, "IQ:");
	if(lpInk){
		char ink[3];
		char *stopstring;

		lpInk += 3;

		memset (ink, 0, 3);
		for (i = 0; i < INK_NUMBER; i++)
		{
			strncpy(ink, lpInk, 2);
			rests[i] = (gint)strtol (ink, &stopstring, 16);
		  
			lpInk +=2;
			if (*lpInk == ';') break;
		}
	}

	lpInk = strstr (lpReply, "INK:");
	if (lpInk)
	{
		lpInk += OFFSET_SEVENTH_INK_CARTRIDGE;
		if (strncmp (lpInk, "9440", 4) == 0)
			inkbox_chenge_table (INKSET_CMYKcmDY);
		
		else if (strncmp (lpInk, "1140", 4) == 0)
			inkbox_chenge_table (INKSET_CMYKcmLk);
	}

	/* ơ */
	lpSts = strstr(lpReply, "ST:");
	if(lpSts){
		strncpy(StsCode, lpSts + 3, 2);
		if(!strcmp(StsCode, "00")){
			/* 顼ơ */
			lpErr = strstr(lpReply, "ER:");
			if(lpErr){
				strncpy(ErrCode, lpErr + 3, 2);
				if(!strcmp(ErrCode, "04")) return STS_ERR_JAM;
				if(!strcmp(ErrCode, "05")) return STS_ERR_NO_INK;
				if(!strcmp(ErrCode, "06")) return STS_ERR_NO_PAPER;
				if(!strcmp(ErrCode, "10")) return STS_ERR_PART_CHANGE;
				if(!strcmp(ErrCode, "11")) return STS_ERR_ROLL;
				if(!strcmp(ErrCode, "12")) return STS_ERR_REPETITION;
				/*  if(!strcmp(ErrCode, "0B")) return STS_ERR_PRINTER; */
			}
			return STS_ERR_FATAL;
		}
		if(!strcmp(StsCode, "01") || !strcmp(StsCode, "07")) return STS_NOT_READY;
		if(!strcmp(StsCode, "03") || !strcmp(StsCode, "02")) return STS_PRINTING;
		if(!strcmp(StsCode, "04")) return STS_IDLE;
		if(!strcmp(StsCode, "09")) return STS_MOTOR_OFF; /* add sk     -Fri Mar  9 2001 */
		if(!strcmp(StsCode, "CN")) return STS_PRINTER_RESET;
	}
	return STS_NO_REPLY;
}

/* ֤ˤäɽ֤Ѳܥ򤹤٤ɽˤ */
static void
SetAllUserButtonHidden ()
{
	GtkWidget *widget;
  
	widget = lookup_widget (_status_str.window, "eject_button");
	gtk_widget_hide (widget);

	widget = lookup_widget (_status_str.window, "continue_button");
	gtk_widget_hide (widget);

	widget = lookup_widget (_status_str.window, "reset_button");
	gtk_widget_hide (widget);

	widget = lookup_widget (_status_str.window, "ink_button");
	gtk_widget_hide (widget);

	widget = lookup_widget (_status_str.window, "cancel_button");
	gtk_widget_set_sensitive (widget, TRUE); /* add sk  - Wed Mar 14 2001 */
	gtk_widget_hide (widget);

	widget = lookup_widget (_status_str.window, "close_button");
	gtk_widget_hide (widget);

	return;
}

/* BOX˾ܺپɽ */
static void
DisplayInformation (GtkText *text, char *info)
{
	/* ֤Ƭˤ */
	gtk_text_set_point (text, 0);
  
	/* ξɽ򥯥ꥢ */
	/*  gtk_text_forward_delete(text, -1); */
	gtk_text_forward_delete (text, gtk_text_get_length (text));
  
	/* ܺپɽ */
	gtk_text_insert (text, NULL, NULL, NULL, info, -1);
  
	/* ưƬ᤹ᡢƬ֤˥ߡʸƺ */
	gtk_text_set_point (text, 0);
	gtk_text_insert (text, NULL, NULL, NULL, " ", -1);
	gtk_text_backward_delete (text, 1);
	return;
}

static int
GetPrinterStatus (gpointer data)
{
	PRINTER_STS status;
	gint rests[INK_NUMBER];
	gint i;
	GtkWidget *widget, *txt, *label;

	/* 󥯻̤ι */
	status = ReadStatuslogfile(rests);
	setInkProg (rests);

	/* Auto down */
	if (status == STS_IDLE)
		auto_down_count (0);	/* count */
	else
		auto_down_count (1);	/* reset */
  
  
	if (status == STS_EKPD_DOWN)
	{
		widget = create_msg_no_ekpd ();
		gtk_widget_show (widget);
		return 0;
	}
  
	if (status == STS_MOTOR_OFF)
		status = _status_str.cur_status;
  
	if (status == _status_str.cur_status)
	{
		/* add sk     - Fri Mar  9 2001 */
		if ((status == STS_ERR_FATAL || status == STS_ERR_JAM)
		    &&  _status_str.stm_st == 2)
		{
			SetAllUserButtonHidden();

			widget = lookup_widget (_status_str.window, "reset_button");
			gtk_widget_show(widget);
			_status_str.stm_st = 0;
		}
      
		/* ˥᡼ι */
		if(_status_str.num_pict == 1)
			return 1;

		_status_str.cur_pict++;

		if(_status_str.cur_pict > _status_str.num_pict - 1)
			_status_str.cur_pict = 0;

		gtk_pixmap_set (GTK_PIXMAP(_status_str.prt_pix),
				_status_str.pixmap[_status_str.cur_pict],
				_status_str.mask[_status_str.cur_pict]);
		return 1;
	}
  
	if (_status_str.cur_status != STS_DEFAULT)
	{
		/* ܤˤPIXMAPλȤ򳰤 */
		for (i = 0; i < _status_str.num_pict; i++)
		{
			gdk_pixmap_unref (_status_str.pixmap[i]);
			gdk_bitmap_unref (_status_str.mask[i]);
		}
	}
  
	/* ܤˤPIXMAPκꡢɽӥܥξ֤ѹ */
	_status_str.cur_pict = 0;
	_status_str.stm_st = 0;

	label = lookup_widget (_status_str.window, "st_label");
	txt = lookup_widget (_status_str.window, "st_txt");

	switch(status)
	{
	case STS_ERR_FATAL:
		_status_str.stm_st = 1;
      
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_ia);
		gtk_label_set_text(GTK_LABEL(label), _("General error"));
		DisplayInformation(GTK_TEXT(txt), _("Delete all print jobs and turn the printer off. "
						    "Remove any foreign objects from inside the printer. "
						    "After a few minutes, turn the printer back on. "
						    "If the problem persists, contact your dealer."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;

	case STS_ERR_JAM:
		_status_str.stm_st = 1;
      
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_i8);
		gtk_label_set_text(GTK_LABEL(label), _("Paper jam"));
		DisplayInformation(GTK_TEXT(txt), _("Press the Maintenance button on the printer or click the Eject button when it appears on the screen. Remove any remaining jammed paper by hand."));
		SetAllUserButtonHidden();

#ifndef NO_ACTION
		widget = lookup_widget (_status_str.window, "eject_button");
		gtk_widget_show(widget);
#endif
		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);

		break;

	case STS_ERR_NO_INK:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_i4);

		gtk_label_set_text(GTK_LABEL(label), _("Ink out"));
		DisplayInformation(GTK_TEXT(txt), _("Ink is out.\n"
						    "Replace the ink cartridge."));

		SetAllUserButtonHidden();

#ifdef INK_CHENGE_OK
		widget = lookup_widget (_status_str.window, "ink_button");
		gtk_widget_show(widget);
#endif

		if (_status_str.cur_status == STS_IDLE || _status_str.cur_status == STS_DEFAULT)
			widget = lookup_widget (_status_str.window, "close_button");
		else
			widget = lookup_widget (_status_str.window, "cancel_button");
		gtk_widget_show(widget);

		break;

	case STS_ERR_NO_PAPER:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_i6);
		gtk_label_set_text(GTK_LABEL(label), _("Paper not loaded correctly"));
		DisplayInformation(GTK_TEXT(txt), _("Reload the paper and press the Maintenance button on the printer or click the Continue button when it appears on the screen. "
						    "To cancel print jobs, click the Cancel button."));
		SetAllUserButtonHidden();

#ifndef NO_ACTION
		widget = lookup_widget (_status_str.window, "continue_button");
		gtk_widget_show(widget);
#endif
		widget = lookup_widget (_status_str.window, "cancel_button");
		gtk_widget_show(widget);

		break;
		/*
		  case STS_ERR_PRINTER:
		  _status_str.num_pict = 2;
		  _status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
		  &_status_str.mask[0], NULL,
		  (gchar**)error);
		  _status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
		  &_status_str.mask[1], NULL,
		  (gchar**)error_ia);
		  gtk_label_set_text(GTK_LABEL(label), _(""));
		  DisplayInformation(GTK_TEXT(txt), _(""));
		  SetAllUserButtonHidden();

		  widget = lookup_widget (_status_str.window, "close_button");
		  gtk_widget_show(widget);
		  break;
		*/

	case STS_ERR_PART_CHANGE:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_ia);
		gtk_label_set_text(GTK_LABEL(label), _("Maintenance call"));
		DisplayInformation(GTK_TEXT(txt), _("Parts inside your printer require maintenance or replacement. "
						    "Contact your dealer."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;
      
	case STS_ERR_ROLL:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_ia);
		gtk_label_set_text(GTK_LABEL(label), _("The paper has fed incorrectly"));
		DisplayInformation(GTK_TEXT(txt), _("Press the roll paper button to return the paper to the correct position."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;
      
	case STS_ERR_REPETITION:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_i6);
		gtk_label_set_text(GTK_LABEL(label), _("Multi-page feed error"));
		DisplayInformation(GTK_TEXT(txt), _("Several pages have been fed into the printer at once. "
						    "Remove and reload the paper, then press the maintenance button."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;
      
	case STS_NOT_READY:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)ready);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)ready_i1);
		gtk_label_set_text(GTK_LABEL(label), _("Printer busy"));
		DisplayInformation(GTK_TEXT(txt), _("The printer is busy cleaning the print head or charging ink. "
						    "Please wait."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;

	case STS_PRINTING:
		auto_down_count_start ();

		_status_str.num_pict = 3;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)printing1);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)printing2);
		_status_str.pixmap[2] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[2], NULL,
								     (gchar**)printing3);
		gtk_label_set_text(GTK_LABEL(label), _("Printing"));
		DisplayInformation(GTK_TEXT(txt), _("Please wait."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "cancel_button");
		gtk_widget_show(widget);

		break;

	case STS_IDLE:
		_status_str.num_pict = 1;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)ready);
		gtk_label_set_text(GTK_LABEL(label), _("Ready"));
		DisplayInformation(GTK_TEXT(txt), _("Ready to print."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;

	case STS_PRINTER_RESET:
		_status_str.stm_st = 1;
      
		_status_str.num_pict = 1;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)warning);
		gtk_label_set_text(GTK_LABEL(label), _("Cancel"));
		DisplayInformation(GTK_TEXT(txt), _("Turn the printer off, and remove any jammed paper. "
						    "After a few minutes, turn the printer back on."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;

	default:
		_status_str.num_pict = 2;
		_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[0], NULL,
								     (gchar**)error);
		_status_str.pixmap[1] = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
								     &_status_str.mask[1], NULL,
								     (gchar**)error_i3);
		gtk_label_set_text(GTK_LABEL(label), _("Communication error"));
		DisplayInformation(GTK_TEXT(txt), _("Check all connections and make sure all devices are on. "
						    "If the power was turned off during printing, cancel the print job. "
						    "If the error does not clear, see your manual."));
		SetAllUserButtonHidden();

		widget = lookup_widget (_status_str.window, "close_button");
		gtk_widget_show(widget);
		break;
	}

	gtk_pixmap_set(GTK_PIXMAP(_status_str.prt_pix),
		       _status_str.pixmap[0],
		       _status_str.mask[0]);
	_status_str.cur_status = status;
	return 1;
}



static int _down_count_start;
static int _down_limit;
static int _down_counter;

void
auto_down_set (int sec)
{
	_down_count_start = 0;
	_down_limit = sec;
	_down_counter = 0;
	return;
}

static void
auto_down_count_start (void)
{
	_down_count_start = 1;
	return;
}

static void
auto_down_count (int flag)
{
	if (_down_limit <= 0 || _down_count_start == 0)
		return;

	if (flag)			/* Refresh */
		_down_counter = 0;
	else
		_down_counter ++;

	if (_down_counter >= _down_limit)
		gtk_main_quit ();

	return;
}

void
init_stm (GtkWidget* window)
{
	/* ɽwidget */
	_status_str.prt_pix = lookup_widget (window, "prt_pix");
  
	_status_str.num_pict = 1;
	_status_str.pixmap[0] = gdk_pixmap_create_from_xpm_d(window->window,
							     &_status_str.mask[0], NULL,
							     (gchar**)warning);
	gtk_pixmap_set(GTK_PIXMAP(_status_str.prt_pix),
		       _status_str.pixmap[0],
		       _status_str.mask[0]);
	gtk_widget_show (_status_str.prt_pix);

	_status_str.window = window;
	_status_str.cur_status = STS_DEFAULT;
	_status_str.cur_pict = 0;
	_status_str.num_pict = 1;
	_status_str.stm_st = 0;
	_status_str.timer = gtk_timeout_add(1000, GetPrinterStatus, NULL);
	_status_str.inkbox_xpm = inkbox_xpm_new ();
	_status_str.inkbox_counter = 0;
	return;
}


void
end_stm (void)
{
	int i;
  
	/* PIXMAPλȤ򳰤 */
	for (i = 0; i < _status_str.num_pict; i++)
	{
		gdk_pixmap_unref (_status_str.pixmap[i]);
		gdk_bitmap_unref (_status_str.mask[i]);
	}

	/* ޡκ */
	gtk_timeout_remove(_status_str.timer);
  
	/* socket */
	sock_close ();
  
	if (_status_str.inkbox_xpm)
		inkbox_xpm_ref (_status_str.inkbox_xpm);

	gtk_main_quit();
}

void
set_cancel_status (void)
{
	GtkWidget* widget;

	/* "顼"̵
	   if (_status_str.stm_st == 1)
	   _status_str.stm_st = 2;
	*/
	widget = lookup_widget (_status_str.window, "cancel_button");
	gtk_widget_set_sensitive (widget, FALSE); /* add sk  - Wed Mar 14 2001 */
	return;
}


static void
setInkProg(gint *rests)
{
	GtkWidget *pix;
	GdkPixmap *inkbox_pixmap;
	GdkBitmap *inkbox_mask;
	int i;
	char pix_name[] = "pixmap?";

	if (rests[0] >= 0 && _status_str.inkbox_use_flag[0] != 0)
	{
		if (_status_str.inkbox_use_flag[_status_str.inkbox_counter] == 0
		    || _status_str.inkbox_counter >= INK_NUMBER)
			_status_str.inkbox_counter = 0;

		pix_name[6] = '1' + _status_str.inkbox_counter;
		pix = lookup_widget (_status_str.window, pix_name);
		
		inkbox_chenge_volume (_status_str.inkbox_xpm, rests[_status_str.inkbox_counter],
				      _status_str.inkbox_counter);
		inkbox_pixmap = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
									    &inkbox_mask, NULL,
									    _status_str.inkbox_xpm);
		gtk_pixmap_set(GTK_PIXMAP(pix),
			       inkbox_pixmap,
			       inkbox_mask);
		gtk_widget_show (pix);
		gdk_pixmap_unref (inkbox_pixmap);
		gdk_bitmap_unref (inkbox_mask);
			
		_status_str.inkbox_use_flag[_status_str.inkbox_counter] = 1;
		_status_str.inkbox_counter++;
	}
	else
	{
		for (i = 0; i < INK_NUMBER; i++)
		{
			pix_name[6] = '1' + i;
			pix = lookup_widget (_status_str.window, pix_name);
			
			if (rests[i] >= 0)
			{
				inkbox_chenge_volume (_status_str.inkbox_xpm, rests[i], i);
				inkbox_pixmap = gdk_pixmap_create_from_xpm_d(_status_str.window->window,
									     &inkbox_mask, NULL,
									     _status_str.inkbox_xpm);
				gtk_pixmap_set(GTK_PIXMAP(pix),
					       inkbox_pixmap,
					       inkbox_mask);
				_status_str.inkbox_use_flag[i] = 1;
				gtk_widget_show (pix);
				gdk_pixmap_unref (inkbox_pixmap);
				gdk_bitmap_unref (inkbox_mask);
			}
			else
			{
				_status_str.inkbox_use_flag[i] = 0;
				gtk_widget_hide (pix);
			}
		}
	}
	return;
};
